# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: shared/sp_shared.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass

import betterproto


class KvAction(betterproto.Enum):
    """
    KVAction is a shared type that is used for protos.KVCommand and
    protos.KVStep. Note that only a subset of actions are used for
    protos.KVCommand (CREATE, UPDATE, DELETE, DELETE_ALL) while protos.KVStep
    uses most of them. protolint:disable:next ENUM_FIELD_NAMES_PREFIX
    """

    KV_ACTION_UNSET = 0
    KV_ACTION_GET = 1
    KV_ACTION_CREATE = 2
    KV_ACTION_UPDATE = 3
    KV_ACTION_EXISTS = 4
    KV_ACTION_DELETE = 5
    KV_ACTION_DELETE_ALL = 6
